use crate::rentry::{build_client, delete_paste, CSRFData, DeleteError};

mod rentry;

const PASTE_NAME: &str = "Hyprland-controversy";

pub struct EditCodeGenerator {
    codes: Vec<String>,
    index: usize,
}

impl EditCodeGenerator {
    pub fn retry_last(&mut self) -> Option<()> {
        if self.index > 0 {
            self.index -= 1;
            Some(())
        } else {
            None
        }
    }
}

impl Default for EditCodeGenerator {
    fn default() -> Self {
        let codes = (b'a'..=b'z')
            .map(|c| (c as char).to_string())
            .chain((b'A'..=b'Z').map(|c| (c as char).to_string()))
            .chain((b'0'..=b'9').map(|c| (c as char).to_string()))
            .collect();

        EditCodeGenerator { codes, index: 0 }
    }
}

impl Iterator for EditCodeGenerator {
    type Item = String;

    fn next(&mut self) -> Option<Self::Item> {
        if self.index < self.codes.len() {
            let code = self.codes[self.index].clone();
            self.index += 1;
            Some(code)
        } else {
            None
        }
    }
}

fn main() {
    let client = build_client();
    let mut csrf_data = CSRFData::get(&client, PASTE_NAME);

    let mut edit_codes = EditCodeGenerator::default();

    while let Some(edit_code) = edit_codes.next() {
        match delete_paste(&client, &csrf_data, &edit_code, PASTE_NAME) {
            Ok(_) => {
                println!("Paste deleted successfully with edit code: {edit_code}");
                return;
            }
            Err(e) => match e {
                DeleteError::FormFieldError(msg) if msg == "Invalid edit code." => {
                    println!("Invalid edit code {edit_code}");
                }
                DeleteError::FormFieldError(msg) => {
                    eprintln!("Form error: {msg}");
                    return;
                }
                DeleteError::FormError(msg)
                    if msg == "You have hit the limit, please wait a bit. e345" =>
                {
                    println!("Hit the limit, waiting...");
                    csrf_data = CSRFData::get(&client, PASTE_NAME);
                    std::thread::sleep(std::time::Duration::from_secs(30));
                    edit_codes.retry_last().unwrap();
                }
                DeleteError::FormError(msg) => {
                    eprintln!("Form error: {msg}");
                    return;
                }
                DeleteError::RequestFailure(msg) => {
                    eprintln!("Request error (non 2XX): {msg}");
                    return;
                }
                DeleteError::UnexpectedRedirect(msg) => {
                    eprintln!("Unexpected redirect: {msg}");
                    return;
                }
                DeleteError::UnknownFailure(msg) => {
                    eprintln!("Unknown failure: {msg}");
                    return;
                }
                DeleteError::ReqwestError(e) => {
                    eprintln!("Request failed with: {e:?}");
                    return;
                }
            },
        }
    }
}
